﻿using FileZillaConfig.Helpers;
using FileZillaConfig.Types;
using System;
using System.Xml.Linq;

namespace FileZillaConfig.ConfigClasses
{
    public class User
    {
        private string _name;
        private string _group;
        private CheckType _enabled;
        private string _comments;
        private Password _password;
        private int _maxConnectionCount;
        private int _limitPerIP;
        private CheckType _bypassUserLimit;
        private CheckType _forceSSL;
        private IPFilter _ipFilter;
        private SpeedLimits _speedLimits;
        private CheckType _bypassServerDownloadLimit;
        private CheckType _bypassServerUploadLimit;
        private Folders _folders;

        #region PROPERTIES
        /// <summary>
        /// User folders
        /// </summary>
        public Folders Folders
        {
            get { return _folders; }
        }


        /// <summary>
        /// Bypass server upload speed limit
        /// </summary>
        public CheckType BypassServerUploadLimit
        {
            get { return _bypassServerUploadLimit; }
            set { _bypassServerUploadLimit = value; }
        }


        /// <summary>
        /// Bypass server download speed limit
        /// </summary>
        public CheckType BypassServerDownloadLimit
        {
            get { return _bypassServerDownloadLimit; }
            set { _bypassServerDownloadLimit = value; }
        }


        /// <summary>
        /// User speed limits
        /// </summary>
        public SpeedLimits SpeedLimits
        {
            get { return _speedLimits; }
        }


        /// <summary>
        /// Filter connection IPs
        /// </summary>
        public IPFilter IPFilter
        {
            get { return _ipFilter; }
        }


        /// <summary>
        /// Force SSL for user login
        /// </summary>
        public CheckType ForceSSL
        {
            get { return _forceSSL; }
            set { _forceSSL = value; }
        }


        /// <summary>
        /// Bypass user limit of server 
        /// </summary>
        public CheckType BypassUserLimit
        {
            get { return _bypassUserLimit; }
            set { _bypassUserLimit = value; }
        }


        /// <summary>
        /// Connection limit per IP
        /// </summary>
        public int ConnectionLimitPerIP
        {
            get { return _limitPerIP; }
            set { _limitPerIP = value; }
        }


        /// <summary>
        /// Maximum connection count
        /// </summary>
        public int MaximumConnection
        {
            get { return _maxConnectionCount; }
            set { _maxConnectionCount = value; }
        }


        /// <summary>
        /// User password
        /// </summary>
        public Password Password
        {
            get { return _password; }
            set { _password = value; }
        }


        /// <summary>
        /// User comments
        /// </summary>
        public string Comments
        {
            get { return _comments; }
            set { _comments = value; }
        }


        /// <summary>
        /// User enabled
        /// </summary>
        public CheckType Enabled
        {
            get { return _enabled; }
            set { _enabled = value; }
        }


        /// <summary>
        /// User group
        /// </summary>
        public string Group
        {
            get { return _group; }
            set { _group = value; }
        }


        /// <summary>
        /// User name
        /// </summary>
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        } 
        #endregion

        public User(string Username)
        {
            this._ipFilter = new IPFilter();
            this._speedLimits = new SpeedLimits();
            this._folders = new Folders();
            this._name = Username;
            this._password = new Password();
        }

        public XElement getXML()
        {
            XElement User = new XElement("User");
            User.Add(new XAttribute("Name", this._name));


            // User options
            User.Add(XMLItem.Option("Pass", this._password.Get));
            User.Add(XMLItem.Option("Group", this._group));
            User.Add(XMLItem.Option("Bypass server userlimit", this._bypassUserLimit));
            User.Add(XMLItem.Option("User Limit", this._maxConnectionCount));
            User.Add(XMLItem.Option("IP Limit", this._limitPerIP));
            User.Add(XMLItem.Option("Enabled", this._enabled));
            User.Add(XMLItem.Option("Comments", this._comments));
            User.Add(XMLItem.Option("ForceSsl", this._forceSSL));

            // User IPFilter
            XElement IpFilter = new XElement("IpFilter");
            IpFilter.Add(new XElement("Disallowed", this._ipFilter.Disallowed));
            IpFilter.Add(new XElement("Allowed", this._ipFilter.Allowed));

            User.Add(IpFilter);

            // User folders
            User.Add(this._folders.getXML());


            // User speed limits
            XElement SpeedLimits = this._speedLimits.getXML();
            SpeedLimits.Add(new XAttribute("DlType", Convert.ToInt16(this._speedLimits.DownloadType).ToString()));
            SpeedLimits.Add(new XAttribute("DlLimit", this._speedLimits.DownloadLimit.ToString()));
            SpeedLimits.Add(new XAttribute("ServerDlLimitBypass", Convert.ToInt16(this._bypassServerDownloadLimit).ToString()));
            SpeedLimits.Add(new XAttribute("UlType", Convert.ToInt16(this._speedLimits.UploadType).ToString()));
            SpeedLimits.Add(new XAttribute("UlLimit", this._speedLimits.UploadLimit.ToString()));
            SpeedLimits.Add(new XAttribute("ServerUlLimitBypass", Convert.ToInt16(this._bypassServerUploadLimit).ToString()));

            User.Add(SpeedLimits);



            return User;
        }

    }
}
